{% load crispy_forms_filters %}
{% load l10n %}

{% for choice in field.field.choices %}
<div class="control">
  <label class="radio" for="id_{{ field.html_name }}_{{ forloop.counter0  }}">
    <input
      type="radio" class="TODO" {% if choice.0 in field.value or choice.0|stringformat:"s" in field.value or choice.0|stringformat:"s" == field.value|default_if_none:""|stringformat:"s" %} checked="checked"{% endif %} name="{{ field.html_name }}" id="id_{{ field.html_name }}_{{ forloop.counter0 }}" value="{{ choice.0|unlocalize }}" {{ field.field.widget.attrs|flatatt }}>
    {{ choice.1|unlocalize }}
  </label>
</div>
{% endfor %}
